@@ -7,6 +7,7 @@ use std::{collections::HashMap, time::Duration};
 use std::{
   rc::Rc,
   sync::{Arc, RwLock},
+  time::Instant,
 };
 
 use crate::{
@@ -17,6 +18,7 @@ use crate::{
   structure::builtin_endpoint::{BuiltinEndpointSet, },
   dds::qos::policy,
 };
+
 use crate::network::udp_listener::UDPListener;
 use crate::network::udp_sender::UDPSender;
 use crate::network::constant::*;
@@ -28,7 +30,6 @@ use crate::discovery::data_types::topic_data::DiscoveredReaderData;
 use crate::discovery::discovery::Discovery;
 
 use crate::{
-  common::timed_event_handler::{TimedEventHandler},
   discovery::discovery_db::DiscoveryDB,
   structure::{dds_cache::DDSCache, topic_kind::TopicKind},
   messages::submessages::submessages::AckSubmessage,
@@ -62,12 +63,11 @@ pub struct DPEventLoop {
   // Adding readers
   add_reader_receiver: TokenReceiverPair<ReaderIngredients>,
   remove_reader_receiver: TokenReceiverPair<GUID>,
-  reader_timed_event_receiver: HashMap<Token, mio_channel::Receiver<TimerMessageType>>,
 
   // Writers
   add_writer_receiver: TokenReceiverPair<WriterIngredients>,
   remove_writer_receiver: TokenReceiverPair<GUID>,
-  writer_timed_event_reciever: HashMap<Token, mio_channel::Receiver<TimerMessageType>>,
+  //writer_timed_event_reciever: HashMap<Token, mio_channel::Receiver<TimerMessageType>>,
 
   stop_poll_receiver: mio_channel::Receiver<()>,
   // GuidPrefix sent in this channel needs to be RTPSMessage source_guid_prefix. Writer needs this to locate RTPSReaderProxy if negative acknack.
@@ -186,10 +186,8 @@ impl DPEventLoop {
       message_receiver: MessageReceiver::new(participant_guid_prefix, acknack_sender),
       add_reader_receiver,
       remove_reader_receiver,
-      reader_timed_event_receiver: HashMap::new(),
       add_writer_receiver,
       remove_writer_receiver,
-      writer_timed_event_reciever: HashMap::new(),
       stop_poll_receiver,
       writers: HashMap::new(),
       ack_nack_reciever: acknack_reciever,
@@ -198,119 +196,130 @@ impl DPEventLoop {
   }
 
   pub fn event_loop(self) {
-    let mut events = Events::with_capacity(8);  // too small capacity just delays events to next poll
+    let mut events = Events::with_capacity(16);  // too small capacity just delays events to next poll
     let mut acknack_timer = mio_extras::timer::Timer::default();
     acknack_timer.set_timeout(PREEMPTIVE_ACKNACK_PERIOD, ());
 
     self.poll
       .register(&acknack_timer, DPEV_ACKNACK_TIMER_TOKEN, Ready::readable(), PollOpt::edge() )
       .unwrap();
-
+    let mut poll_alive = Instant::now();
     let mut ev_wrapper = self;
     loop {
-      ev_wrapper.poll.poll(&mut events, None)
+      ev_wrapper.poll.poll(&mut events, Some(Duration::from_millis(2000)))
         .expect("Failed in waiting of poll.");
 
-      for event in events.iter() {
-        match EntityId::from_token( event.token() ) {
-          TokenDecode::FixedToken(fixed_token) =>
-            match fixed_token {
-              STOP_POLL_TOKEN => {
-                info!("Stopping dp_event_loop");
-                return
-              }
-              DISCOVERY_LISTENER_TOKEN |
-              DISCOVERY_MUL_LISTENER_TOKEN |
-              USER_TRAFFIC_LISTENER_TOKEN |
-              USER_TRAFFIC_MUL_LISTENER_TOKEN => {
-                let udp_messages = ev_wrapper.udp_listeners.get_mut(&event.token())
-                  .map_or_else(
-                      | | { error!("No listener with token {:?}", &event.token() ); vec![] }, 
-                      |l| l.get_messages()
-                    );
-                for packet in udp_messages.into_iter() {
-                  ev_wrapper.message_receiver.handle_received_packet(packet)
+      // liveness watchdog
+      let now = Instant::now();
+      if now > poll_alive + Duration::from_secs(2) {
+        info!("Poll loop alive");
+        poll_alive = now;
+      }
+
+
+      if events.is_empty() {
+        info!("dp_event_loop idling.")
+      } else {
+        for event in events.iter() {
+          match EntityId::from_token( event.token() ) {
+            TokenDecode::FixedToken(fixed_token) =>
+              match fixed_token {
+                STOP_POLL_TOKEN => {
+                  info!("Stopping dp_event_loop");
+                  return
                 }
-              }
-              ADD_READER_TOKEN | REMOVE_READER_TOKEN => {
-                ev_wrapper.handle_reader_action(&event);
-              }
-              ADD_WRITER_TOKEN | REMOVE_WRITER_TOKEN => {
-                ev_wrapper.handle_writer_action(&event); 
-              }
-              ACKNACK_MESSGAGE_TO_LOCAL_WRITER_TOKEN => {
-                ev_wrapper.handle_writer_acknack_action(&event);
-              }
-              DISCOVERY_UPDATE_NOTIFICATION_TOKEN => {
-                while let Ok(dnt) = ev_wrapper.discovery_update_notification_receiver.try_recv() {
-                  use DiscoveryNotificationType::*;
-                  match dnt {
-                    WriterUpdated{ discovered_writer_data } => 
-                      ev_wrapper.remote_writer_discovered(discovered_writer_data),
+                DISCOVERY_LISTENER_TOKEN |
+                DISCOVERY_MUL_LISTENER_TOKEN |
+                USER_TRAFFIC_LISTENER_TOKEN |
+                USER_TRAFFIC_MUL_LISTENER_TOKEN => {
+                  let udp_messages = ev_wrapper.udp_listeners.get_mut(&event.token())
+                    .map_or_else(
+                        | | { error!("No listener with token {:?}", &event.token() ); vec![] }, 
+                        |l| l.get_messages()
+                      );
+                  for packet in udp_messages.into_iter() {
+                    ev_wrapper.message_receiver.handle_received_packet(packet)
+                  }
+                }
+                ADD_READER_TOKEN | REMOVE_READER_TOKEN => {
+                  ev_wrapper.handle_reader_action(&event);
+                }
+                ADD_WRITER_TOKEN | REMOVE_WRITER_TOKEN => {
+                  ev_wrapper.handle_writer_action(&event); 
+                }
+                ACKNACK_MESSGAGE_TO_LOCAL_WRITER_TOKEN => {
+                  ev_wrapper.handle_writer_acknack_action(&event);
+                }
+                DISCOVERY_UPDATE_NOTIFICATION_TOKEN => {
+                  while let Ok(dnt) = ev_wrapper.discovery_update_notification_receiver.try_recv() {
+                    use DiscoveryNotificationType::*;
+                    match dnt {
+                      WriterUpdated{ discovered_writer_data } => 
+                        ev_wrapper.remote_writer_discovered(discovered_writer_data),
 
-                    WriterLost{writer_guid} => ev_wrapper.remote_writer_lost(writer_guid) ,
+                      WriterLost{writer_guid} => ev_wrapper.remote_writer_lost(writer_guid) ,
 
-                    ReaderUpdated{ discovered_reader_data, rtps_reader_proxy, _needs_new_cache_change } => 
-                      ev_wrapper.remote_reader_discovered(discovered_reader_data, rtps_reader_proxy, _needs_new_cache_change),
+                      ReaderUpdated{ discovered_reader_data, rtps_reader_proxy, _needs_new_cache_change } => 
+                        ev_wrapper.remote_reader_discovered(discovered_reader_data, rtps_reader_proxy, _needs_new_cache_change),
 
-                    ReaderLost{ reader_guid } => ev_wrapper.remote_reader_lost(reader_guid) ,
+                      ReaderLost{ reader_guid } => ev_wrapper.remote_reader_lost(reader_guid) ,
 
-                    ParticipantUpdated{ guid_prefix } => ev_wrapper.update_participant(guid_prefix),
-                      
-                    ParticipantLost{ guid_prefix } => ev_wrapper.remote_participant_lost(guid_prefix),
+                      ParticipantUpdated{ guid_prefix } => ev_wrapper.update_participant(guid_prefix),
+                        
+                      ParticipantLost{ guid_prefix } => ev_wrapper.remote_participant_lost(guid_prefix),
 
-                    TopicsInfoUpdated => ev_wrapper.update_topics(),
-                    AssertTopicLiveliness{ writer_guid , manual_assertion } => {
-                      ev_wrapper.writers.get_mut(&writer_guid.entityId)
-                        .map( |w| w.handle_heartbeat_tick(manual_assertion) ); 
+                      TopicsInfoUpdated => ev_wrapper.update_topics(),
+                      AssertTopicLiveliness{ writer_guid , manual_assertion } => {
+                        ev_wrapper.writers.get_mut(&writer_guid.entityId)
+                          .map( |w| w.handle_heartbeat_tick(manual_assertion) ); 
+                      }
                     }
-                  }
-                }              
-              }
-              DPEV_ACKNACK_TIMER_TOKEN => {
-                ev_wrapper.message_receiver.send_preemptive_acknacks();
-                acknack_timer.set_timeout(PREEMPTIVE_ACKNACK_PERIOD, ());
-              }
+                  }              
+                }
+                DPEV_ACKNACK_TIMER_TOKEN => {
+                  ev_wrapper.message_receiver.send_preemptive_acknacks();
+                  acknack_timer.set_timeout(PREEMPTIVE_ACKNACK_PERIOD, ());
+                }
 
-              fixed_unknown => {
-                error!("Unknown event.token {:?} = 0x{:x?} , decoded as {:?}", 
-                  event.token(), event.token().0, fixed_unknown );
+                fixed_unknown => {
+                  error!("Unknown event.token {:?} = 0x{:x?} , decoded as {:?}", 
+                    event.token(), event.token().0, fixed_unknown );
+                }
+              },
+
+            // Commands/actions
+            TokenDecode::Entity( eid ) => 
+              if eid.kind().is_reader() {
+                ev_wrapper.message_receiver.get_reader_mut( eid )
+                  .map( |reader| reader.process_command() )
+                  .unwrap_or_else(|| error!("Event for unknown reader {:?}",eid));
+              } else if eid.kind().is_writer() {
+                let local_readers =
+                  match ev_wrapper.writers.get_mut( &eid ) {
+                    None => { error!("Event for unknown writer {:?}",eid); vec![] },
+                    Some(writer) => {
+                      // Writer will record data to DDSCache and send it out.
+                      writer.process_writer_command();
+                      writer.get_local_readers()
+                    }
+                  };
+                // Notify local (same participant) readers that new data is available in the cache.
+                ev_wrapper.message_receiver.notify_data_to_readers(local_readers);
+              } else { 
+                error!("Entity Event for unknown EntityKind {:?}",eid); 
+              },
+              
+            // Timed Actions
+            TokenDecode::AltEntity( eid ) =>
+              if eid.kind().is_reader() { 
+                ev_wrapper.handle_reader_timed_event(eid); 
+              } else if eid.kind().is_writer() {
+                ev_wrapper.handle_writer_timed_event(eid);
               }
-            },
-
-          // Commands/actions
-          TokenDecode::Entity( eid ) => 
-            if eid.kind().is_reader() {
-              ev_wrapper.message_receiver.get_reader_mut( eid )
-                .map( |reader| reader.process_command() )
-                .unwrap_or_else(|| error!("Event for unknown reader {:?}",eid));
-            } else if eid.kind().is_writer() {
-              let local_readers =
-                match ev_wrapper.writers.get_mut( &eid ) {
-                  None => { error!("Event for unknown writer {:?}",eid); vec![] },
-                  Some(writer) => {
-                    // Writer will record data to DDSCache and send it out.
-                    writer.process_writer_command();
-                    writer.get_local_readers()
-                  }
-                };
-              // Notify local (same participant) readers that new data is available in the cache.
-              ev_wrapper.message_receiver.notify_data_to_readers(local_readers);
-            } else { 
-              error!("Entity Event for unknown EntityKind {:?}",eid); 
-            },
-            
-          // Timed Actions
-          TokenDecode::AltEntity( eid ) =>
-            if eid.kind().is_reader() { 
-              ev_wrapper.handle_reader_timed_event(&event); 
-            } else if eid.kind().is_writer() {
-              ev_wrapper.handle_writer_timed_event(&event);
-            }
-            else { error!("AltEntity Event for unknown EntityKind {:?}",eid); },
-        }
-      } // for    
-
+              else { error!("AltEntity Event for unknown EntityKind {:?}",eid); },
+          }
+        } // for    
+      } // if
     } // loop
   } // fn
 
@@ -319,27 +328,27 @@ impl DPEventLoop {
       ADD_READER_TOKEN => {
         trace!("add reader(s)");
         while let Ok(new_reader_ing) = self.add_reader_receiver.receiver.try_recv() {
-          let (timed_action_sender, timed_action_receiver) =
-            mio_channel::sync_channel::<TimerMessageType>(10);
 
-          let mut new_reader = 
-            Reader::new(new_reader_ing, self.ddscache.clone(), self.udp_sender.clone());
-
-          let time_handler: TimedEventHandler = TimedEventHandler::new(timed_action_sender.clone());
-          new_reader.add_timed_event_handler(time_handler);
-          // Timed action polling
+          let timer = mio_extras::timer::Builder::default()
+            .num_slots(8)
+            .build();
           self
             .poll
             .register(
-              &timed_action_receiver,
-              new_reader.get_reader_alt_entity_token(),
+              &timer,
+              new_reader_ing.guid.entityId.as_alt_token(),
               Ready::readable(),
               PollOpt::edge(),
             )
             .expect("Reader timer channel registeration failed!");
-          self
-            .reader_timed_event_receiver
-            .insert(new_reader.get_reader_alt_entity_token(), timed_action_receiver);
+          let mut new_reader = 
+            Reader::new(
+              new_reader_ing, 
+              self.ddscache.clone(), 
+              self.udp_sender.clone(),
+              timer,
+              );
+
           // Non-timed action polling
           self
             .poll
@@ -359,13 +368,8 @@ impl DPEventLoop {
       REMOVE_READER_TOKEN => {
         while let Ok(old_reader_guid) = self.remove_reader_receiver.receiver.try_recv() {
           if let Some(old_reader) = self.message_receiver.remove_reader(old_reader_guid) {
-            if let Some(receiver) = 
-              self.reader_timed_event_receiver.remove(&old_reader.get_reader_alt_entity_token()) {
-                self.poll.deregister(&receiver)
-                  .unwrap_or_else(|e| error!("reader_timed_event_receiver deregister: {:?}",e));
-            } else {
-              warn!("Reader had no reader_timed_event_receiver? {:?}", old_reader_guid);
-            }
+            self.poll.deregister( &old_reader.timed_event_timer )
+              .unwrap_or_else(|e| error!("Cannot deregister Reader timed_event_timer: {:?}",e));
             self.poll.deregister( &old_reader.data_reader_command_receiver )
               .unwrap_or_else(|e| error!("Cannot deregister data_reader_command_receiver: {:?}",e));
           } else {
@@ -381,8 +385,21 @@ impl DPEventLoop {
     match event.token() {
       ADD_WRITER_TOKEN => {
         while let Ok(new_writer_ingredients) = self.add_writer_receiver.receiver.try_recv() {
-          let mut new_writer = Writer::new(
-            new_writer_ingredients, self.ddscache.clone(), self.udp_sender.clone() );
+          let timer = mio_extras::timer::Builder::default()
+            .num_slots(8)
+            .build();
+          self.poll.register(
+              &timer,
+              new_writer_ingredients.guid.entityId.as_alt_token(),
+              Ready::readable(),
+              PollOpt::edge(),
+            )
+            .expect("Writer heartbeat timer channel registration failed!!");
+          let new_writer = Writer::new(
+            new_writer_ingredients,
+            self.ddscache.clone(),
+            self.udp_sender.clone(),
+            timer );
 
           self.poll.register(
             &new_writer.writer_command_receiver,
@@ -390,38 +407,16 @@ impl DPEventLoop {
             Ready::readable(),
             PollOpt::edge(),
           ).expect("Writer command channel registration failed!!");
-          let (timed_action_sender, timed_action_receiver) =
-            mio_channel::sync_channel::<TimerMessageType>(10);
-          let time_handler: TimedEventHandler = TimedEventHandler::new(timed_action_sender.clone());
-          new_writer.add_timed_event_handler(time_handler);
-
-          self.poll.register(
-              &timed_action_receiver,
-              new_writer.get_timed_event_entity_token(),
-              Ready::readable(),
-              PollOpt::edge(),
-            )
-            .expect("Writer heartbeat timer channel registration failed!!");
-          self.writer_timed_event_reciever.insert(
-            new_writer.get_timed_event_entity_token(),
-            timed_action_receiver,
-          );
           self.writers.insert(new_writer.get_guid().entityId, new_writer);
         }
       }
       REMOVE_WRITER_TOKEN => {
         while let Ok(writer_guid) = &self.remove_writer_receiver.receiver.try_recv() {
-          let writer = self.writers.remove(&writer_guid.entityId);
-          
-          if let Some(w) = writer {
+          if let Some(w) = self.writers.remove(&writer_guid.entityId) {
             self.poll.deregister(&w.writer_command_receiver)
-              .unwrap_or_else( |e| error!("Deregister fail {:?}",e));
-
-            if let Some(rec) = 
-                self.writer_timed_event_reciever.remove( &w.get_timed_event_entity_token() ) {
-              self.poll.deregister(&rec)
-                .unwrap_or_else( |e| error!("Deregister fail {:?}",e));
-            }
+              .unwrap_or_else( |e| error!("Deregister fail (writer command rec) {:?}",e));
+            self.poll.deregister(&w.timed_event_timer)
+              .unwrap_or_else( |e| error!("Deregister fail (writer timer) {:?}",e));
           }
         }
       }
@@ -431,29 +426,21 @@ impl DPEventLoop {
 
   /// Writer timed events can be heatrbeats or cache cleaning events.
   /// events are distinguished by TimerMessageType which is send via mio channel. Channel token in
-  fn handle_writer_timed_event(&mut self, event: &Event) {
-    let reciever = self.writer_timed_event_reciever.get(&event.token())
-      .expect("Did not find a heartbeat receiver");
-    while let Ok(timer_message) = reciever.try_recv() {
-      self.writers.iter_mut()
-          .find(|(_guid,writer)| writer.get_timed_event_entity_token() == event.token())
-          .map( |(_guid, writer)| writer.handle_timed_event(timer_message));
+  fn handle_writer_timed_event(&mut self, entity_id: EntityId) {
+    match self.writers.get_mut(&entity_id) {
+      Some(writer) => {
+        writer.handle_timed_event()
+      }
+      None => error!("Writer was not found with {:?}", entity_id),
     }
   }
 
-  fn handle_reader_timed_event(&mut self, event: &Event) {
-    let reciever = self.reader_timed_event_receiver.get(&event.token())
-      .expect("Did not found reader timed event reciever!");
-    // TODO: Why do we have separate message channels for reader and writer, if they have contain the same data type?
-
-    while let Ok(timer_message) = reciever.try_recv() {
-      match self.message_receiver.available_readers.values_mut()
-          .find(|reader| reader.get_entity_token() == event.token()) {
-        Some(reader) => {
-          reader.handle_timed_event(timer_message);
-        }
-        None => error!("Reader was not found with entity token {:?}",  event.token()),
+  fn handle_reader_timed_event(&mut self, entity_id: EntityId) {
+    match self.message_receiver.get_reader_mut(entity_id) {
+      Some(reader) => {
+        reader.handle_timed_event()
       }
+      None => error!("Reader was not found with {:?}", entity_id),
     }
   }
 
