@@ -1,6 +1,5 @@
 use std::{
-  collections::{btree_map::Iter as BTreeIter, HashMap, BTreeMap},
-  iter::Map,
+  collections::{HashMap, BTreeMap},
   time::Instant,
 };
 
@@ -17,7 +16,8 @@ use crate::structure::{guid::GUID, guid::EntityId,
 
 use crate::{
   dds::{
-    rtps_reader_proxy::RtpsReaderProxy, reader::Reader, participant::DomainParticipant,
+    rtps_reader_proxy::RtpsReaderProxy, reader::{ReaderIngredients,}, 
+    participant::DomainParticipant,
     topic::Topic, traits::TopicDescription,
   },
 };
@@ -75,13 +75,16 @@ impl DiscoveryDB {
     }
   }
 
-  pub fn update_participant(&mut self, data: &SPDPDiscoveredParticipantData) {
+  // Returns if particiapnt was previously unkonwn
+  pub fn update_participant(&mut self, data: &SPDPDiscoveredParticipantData) -> bool {
       debug!("update_participant: {:?}",&data);
       let guid = data.participant_guid;
+
       // sanity check
       if guid.entityId != EntityId::ENTITYID_PARTICIPANT {
         error!("Discovered participant GUID entity_id is not for participant: {:?}",guid);
-        return // Maybe we should discard the participant here?
+        // Maybe we should discard the participant here?
+        return false 
       }
 
       // We allow discovery to discover self, since our discovery readers
@@ -94,16 +97,21 @@ impl DiscoveryDB {
       //   return
       // }
 
+      let mut new_participant = false;
       if self.participant_proxies.get( &guid.guidPrefix ).is_none() {
         info!("New remote participant: {:?}", &data);
+        new_participant = true;
         if guid == self.my_guid {
           info!("Remote participant {:?} is myself, but some reflection is good.",
             guid);
+          new_participant = false;
         }
       }
       // actual work here:
       self.participant_proxies.insert(guid.guidPrefix, data.clone());
       self.participant_last_life_signs.insert(guid.guidPrefix, Instant::now() );
+
+      new_participant
   }
 
   pub fn remove_participant(&mut self, guid_prefix: GuidPrefix) {
@@ -131,7 +139,7 @@ impl DiscoveryDB {
     let to_remove :Vec<GUID> = 
           self.external_topic_readers
             .range( guid_prefix.range() )
-            .map(|(g,_)| g.clone() )
+            .map(|(g,_)| *g )
             .collect();
     for guid in to_remove {
       self.external_topic_readers.remove(&guid);
@@ -139,6 +147,7 @@ impl DiscoveryDB {
   }
 
   pub fn remove_topic_reader(&mut self, guid: GUID) {
+    info!("remove_topic_reader {:?}",guid);
     self.external_topic_readers.remove(&guid);
   }
 
@@ -147,7 +156,7 @@ impl DiscoveryDB {
     let to_remove :Vec<GUID> = 
           self.external_topic_writers
             .range( guid_prefix.range() )
-            .map(|(g,_)| g.clone() )
+            .map(|(g,_)| *g )
             .collect();
     for guid in to_remove {
       self.external_topic_writers.remove(&guid);
@@ -175,9 +184,9 @@ impl DiscoveryDB {
           // keep, if duration not exeeded
           let elapsed = Duration::from_std(inow.duration_since(last_life));
           if elapsed <= lease_duration {
-            () // this is a keeper
+            // this is a keeper
           } else {
-            debug!("participant cleanup - deleting participant proxy {:?}. lease_duration = {:?} elapsed = {:?}",
+            info!("participant cleanup - deleting participant proxy {:?}. lease_duration = {:?} elapsed = {:?}",
                   guid, lease_duration, elapsed);
             to_remove.push(guid);    
           }
@@ -187,49 +196,43 @@ impl DiscoveryDB {
         }
       } // match
     } // for
-    for guid in &to_remove {
-      self.participant_proxies.remove(guid).expect("Removing fail 1");
-      self.participant_last_life_signs.remove(guid).expect("Removing fail 2");
+    for guid in to_remove.iter() {
+      self.remove_participant(*guid);
     }
-
     to_remove
   }
 
-  fn topic_has_writers_or_readers(&self, topic_name: &String) -> bool {
+  fn topic_has_writers_or_readers(&self, topic_name: &str) -> bool {
     // TODO: This entire function has silly implementation.
     // We should really have a separate map from Topic to Readers & Writers
-    if let Some(_) =
-      self
+    if self
         .local_topic_readers
         .iter()
-        .find(|(_, p)| p.subscription_topic_data.topic_name() == topic_name)
+        .any(|(_, p)| p.subscription_topic_data.topic_name() == topic_name)
     {
       return true
     }
 
-    if let Some(_) =
-      self
+    if self
         .local_topic_writers
         .iter()
-        .find(|(_, p)| &p.publication_topic_data.topic_name == topic_name)
+        .any(|(_, p)| p.publication_topic_data.topic_name == topic_name)
     {
       return true
     }
 
-    if let Some(_) =
-      self
+    if self
         .external_topic_readers
         .values()
-        .find(|p| p.subscription_topic_data.topic_name() == topic_name)
+        .any(|p| p.subscription_topic_data.topic_name() == topic_name)
     {
       return true
     }
 
-    if let Some(_) =
-      self
+    if self
         .external_topic_writers
         .values()
-        .find(|p| &p.publication_topic_data.topic_name == topic_name)
+        .any(|p| p.publication_topic_data.topic_name == topic_name)
     {
       return true
     }
@@ -243,29 +246,15 @@ impl DiscoveryDB {
       .topics
       .iter()
       .map(|(tn, _)| tn)
-      .filter(|tn| !self.topic_has_writers_or_readers(tn))
-      .map(|tn| tn.clone())
+      .filter(|tn| !self.topic_has_writers_or_readers(tn)).cloned()
       .collect();
     for dt in dead_topics.iter() {
       self.topics.remove(dt);
     }
   }
 
-  // Please explain how this works.
-  pub fn get_participants<'a>(&'a self) -> Map<
-    BTreeIter<'a, GuidPrefix, SPDPDiscoveredParticipantData>,
-    fn((&GuidPrefix, &'a SPDPDiscoveredParticipantData)) -> &'a SPDPDiscoveredParticipantData,
-  > {
-    type cvfun<'a> =
-      fn((&GuidPrefix, &'a SPDPDiscoveredParticipantData)) -> &'a SPDPDiscoveredParticipantData;
-    fn conver<'a>(
-      (_, data): (&GuidPrefix, &'a SPDPDiscoveredParticipantData),
-    ) -> &'a SPDPDiscoveredParticipantData {
-      data
-    }
-    let cv: cvfun = conver;
-    let a = self.participant_proxies.iter().map(cv);
-    a
+  pub fn get_participants(&self) -> impl Iterator<Item = &SPDPDiscoveredParticipantData> {
+    self.participant_proxies.values()
   }
 
   pub fn update_local_topic_writer(&mut self, writer: DiscoveredWriterData) {
@@ -286,7 +275,8 @@ impl DiscoveryDB {
     self.external_topic_writers.values()
   }
 
-  
+  // TODO: This is silly. Returns one of the paramters cloned, or None
+  // TODO: Why are we here checking if discovery db already has this? What about reader proxies in writers?
   pub fn update_subscription(&mut self, data: &DiscoveredReaderData) -> Option<(DiscoveredReaderData, RtpsReaderProxy)> {
     let guid = data.reader_proxy.remote_reader_guid;
     // we could return None to indicate that we already knew all about this reader
@@ -318,6 +308,7 @@ impl DiscoveryDB {
     }
   }
 
+  // TODO: This is silly. Returns one of the paramters cloned, or None
   pub fn update_publication(&mut self, data: &DiscoveredWriterData) -> Option<DiscoveredWriterData> {
     match self.external_topic_writers.get(&data.writer_proxy.remote_writer_guid) {
       Some(dwd) if dwd == data => None , // already up to date
@@ -335,17 +326,17 @@ impl DiscoveryDB {
       key: None,
       name: drd.subscription_topic_data.topic_name().clone(),
       type_name: drd.subscription_topic_data.type_name().clone(),
-      durability: drd.subscription_topic_data.durability().clone(),
-      deadline: drd.subscription_topic_data.deadline().clone(),
-      latency_budget: drd.subscription_topic_data.latency_budget().clone(),
-      liveliness: drd.subscription_topic_data.liveliness().clone(),
-      reliability: drd.subscription_topic_data.reliability().clone(),
-      lifespan: drd.subscription_topic_data.lifespan().clone(),
-      destination_order: drd.subscription_topic_data.destination_order().clone(),
-      presentation: drd.subscription_topic_data.presentation().clone(),
+      durability: *drd.subscription_topic_data.durability(),
+      deadline: *drd.subscription_topic_data.deadline(),
+      latency_budget: *drd.subscription_topic_data.latency_budget(),
+      liveliness: *drd.subscription_topic_data.liveliness(),
+      reliability: *drd.subscription_topic_data.reliability(),
+      lifespan: *drd.subscription_topic_data.lifespan(),
+      destination_order: *drd.subscription_topic_data.destination_order(),
+      presentation: *drd.subscription_topic_data.presentation(),
       history: None,
       resource_limits: None,
-      ownership: drd.subscription_topic_data.ownership().clone(),
+      ownership: *drd.subscription_topic_data.ownership(),
     });
 
     self.update_topic_data(&topic_data);
@@ -356,17 +347,17 @@ impl DiscoveryDB {
       key: None,
       name: dwd.publication_topic_data.topic_name.clone(),
       type_name: dwd.publication_topic_data.type_name.clone(),
-      durability: dwd.publication_topic_data.durability.clone(),
-      deadline: dwd.publication_topic_data.deadline.clone(),
-      latency_budget: dwd.publication_topic_data.latency_budget.clone(),
-      liveliness: dwd.publication_topic_data.liveliness.clone(),
-      reliability: dwd.publication_topic_data.reliability.clone(),
-      lifespan: dwd.publication_topic_data.lifespan.clone(),
-      destination_order: dwd.publication_topic_data.destination_order.clone(),
-      presentation: dwd.publication_topic_data.presentation.clone(),
+      durability: dwd.publication_topic_data.durability,
+      deadline: dwd.publication_topic_data.deadline,
+      latency_budget: dwd.publication_topic_data.latency_budget,
+      liveliness: dwd.publication_topic_data.liveliness,
+      reliability: dwd.publication_topic_data.reliability,
+      lifespan: dwd.publication_topic_data.lifespan,
+      destination_order: dwd.publication_topic_data.destination_order,
+      presentation: dwd.publication_topic_data.presentation,
       history: None,
       resource_limits: None,
-      ownership: dwd.publication_topic_data.ownership.clone(),
+      ownership: dwd.publication_topic_data.ownership,
     });
 
     self.update_topic_data(&topic_data);
@@ -375,19 +366,19 @@ impl DiscoveryDB {
   pub fn update_topic_data_p(&mut self, topic: &Topic) {
     let topic_data = DiscoveredTopicData::new(TopicBuiltinTopicData {
       key: None,
-      name: String::from(topic.get_name()),
+      name: topic.get_name(),
       type_name: String::from(topic.get_type().name()),
-      durability: topic.get_qos().durability.clone(),
-      deadline: topic.get_qos().deadline.clone(),
-      latency_budget: topic.get_qos().latency_budget.clone(),
-      liveliness: topic.get_qos().liveliness.clone(),
-      reliability: topic.get_qos().reliability.clone(),
-      lifespan: topic.get_qos().lifespan.clone(),
-      destination_order: topic.get_qos().destination_order.clone(),
-      presentation: topic.get_qos().presentation.clone(),
-      history: topic.get_qos().history.clone(),
-      resource_limits: topic.get_qos().resource_limits.clone(),
-      ownership: topic.get_qos().ownership.clone(),
+      durability: topic.get_qos().durability,
+      deadline: topic.get_qos().deadline,
+      latency_budget: topic.get_qos().latency_budget,
+      liveliness: topic.get_qos().liveliness,
+      reliability: topic.get_qos().reliability,
+      lifespan: topic.get_qos().lifespan,
+      destination_order: topic.get_qos().destination_order,
+      presentation: topic.get_qos().presentation,
+      history: topic.get_qos().history,
+      resource_limits: topic.get_qos().resource_limits,
+      ownership: topic.get_qos().ownership,
     });
 
     self.update_topic_data(&topic_data);
@@ -413,9 +404,9 @@ impl DiscoveryDB {
     &mut self,
     domain_participant: &DomainParticipant,
     topic: &Topic,
-    reader: &Reader,
+    reader: &ReaderIngredients,
   ) {
-    let reader_guid = reader.get_guid();
+    let reader_guid = reader.guid;
 
     let reader_proxy = RtpsReaderProxy::from_reader(
       reader,
@@ -426,7 +417,7 @@ impl DiscoveryDB {
     let mut subscription_data = SubscriptionBuiltinTopicData::new(
       reader_guid,
       &topic.get_name(),
-      &topic.get_type().name(),
+      topic.get_type().name(),
       &topic.get_qos(),
     );
     subscription_data.set_participant_key(domain_participant.get_guid());
@@ -468,19 +459,19 @@ impl DiscoveryDB {
     self.writers_updated = updated;
   }
 
-  pub fn get_all_local_topic_readers<'a>(
-    &'a self,
-  ) -> impl Iterator<Item = &'a DiscoveredReaderData> {
+  pub fn get_all_local_topic_readers(
+    &self,
+  ) -> impl Iterator<Item = &DiscoveredReaderData> {
     self.local_topic_readers.iter().map(|(_, p)| p)
   }
 
-  pub fn get_all_local_topic_writers<'a>(
-    &'a self,
-  ) -> impl Iterator<Item = &'a DiscoveredWriterData> {
+  pub fn get_all_local_topic_writers(
+    &self,
+  ) -> impl Iterator<Item = &DiscoveredWriterData> {
     self.local_topic_writers.iter().map(|(_, p)| p)
   }
 
-  pub fn get_all_topics<'a>(&'a self) -> impl Iterator<Item = &'a DiscoveredTopicData> {
+  pub fn get_all_topics(&self) -> impl Iterator<Item = &DiscoveredTopicData> {
     self.topics.iter()
       .filter(|(s, _)| !s.starts_with("DCPS"))
       .map(|(_, v)| v)
@@ -528,7 +519,10 @@ mod tests {
     },
   };
   use std::sync::{RwLock, Arc};
+  use std::rc::Rc;
 
+  use crate::network::udp_sender::UDPSender;
+  use crate::dds::reader::Reader;
   use crate::structure::guid::*;
   use crate::serialization::cdr_serializer::CDRSerializerAdapter;
   use byteorder::LittleEndian;
@@ -673,39 +667,49 @@ mod tests {
     let (_reader_commander2, reader_command_receiver2) =
       mio_extras::channel::sync_channel::<ReaderCommand>(100);
 
-    let reader = Reader::new(
-      GUID::dummy_test_guid(EntityKind::READER_NO_KEY_USER_DEFINED),
-      notification_sender.clone(),
-      status_sender.clone(),
+    let reader_ing = ReaderIngredients {
+      guid: GUID::dummy_test_guid(EntityKind::READER_NO_KEY_USER_DEFINED),
+      notification_sender: notification_sender.clone(),
+      status_sender: status_sender.clone(),
+      topic_name: topic.get_name().to_string(),
+      qos_policy: QosPolicies::qos_none(),
+      data_reader_command_receiver: reader_command_receiver1,
+    };
+
+    discoverydb.update_local_topic_reader(&dp, &topic, &reader_ing);
+    assert_eq!(discoverydb.local_topic_readers.len(), 1);
+    assert_eq!(discoverydb.get_local_topic_readers(&topic).len(), 1);
+
+    discoverydb.update_local_topic_reader(&dp, &topic, &reader_ing);
+    assert_eq!(discoverydb.local_topic_readers.len(), 1);
+    assert_eq!(discoverydb.get_local_topic_readers(&topic).len(), 1);
+
+    let _reader = Reader::new(
+      reader_ing,
       Arc::new(RwLock::new(DDSCache::new())),
-      topic.get_name().to_string(),
-      QosPolicies::qos_none(),
-      reader_command_receiver1,
+      Rc::new(UDPSender::new(0).unwrap()),
     );
 
-    discoverydb.update_local_topic_reader(&dp, &topic, &reader);
-    assert_eq!(discoverydb.local_topic_readers.len(), 1);
-    assert_eq!(discoverydb.get_local_topic_readers(&topic).len(), 1);
-
-    discoverydb.update_local_topic_reader(&dp, &topic, &reader);
-    assert_eq!(discoverydb.local_topic_readers.len(), 1);
-    assert_eq!(discoverydb.get_local_topic_readers(&topic).len(), 1);
-
-    let reader = Reader::new(
-      GUID::new_with_prefix_and_id(GuidPrefix::new(b"Another fake"), EntityId {
+    let reader_ing = ReaderIngredients {
+      guid: GUID::new_with_prefix_and_id(GuidPrefix::new(b"Another fake"), EntityId {
         entityKey: [1, 2, 3],
         entityKind: EntityKind::READER_NO_KEY_USER_DEFINED
       }), // GUID needs to be different in order to be added
-      notification_sender.clone(),
-      status_sender.clone(),
-      Arc::new(RwLock::new(DDSCache::new())),
-      topic.get_name().to_string(),
-      QosPolicies::qos_none(),
-      reader_command_receiver2,
-    );
+      notification_sender: notification_sender.clone(),
+      status_sender: status_sender.clone(),
+      topic_name: topic.get_name().to_string(),
+      qos_policy: QosPolicies::qos_none(),
+      data_reader_command_receiver: reader_command_receiver2,
+    };
 
-    discoverydb.update_local_topic_reader(&dp, &topic, &reader);
+    discoverydb.update_local_topic_reader(&dp, &topic, &reader_ing);
     assert_eq!(discoverydb.get_local_topic_readers(&topic).len(), 2);
     assert_eq!(discoverydb.get_all_local_topic_readers().count(), 2);
+
+    let _reader = Reader::new(
+      reader_ing,
+      Arc::new(RwLock::new(DDSCache::new())),
+      Rc::new(UDPSender::new(0).unwrap()),
+    );
   }
 }
