@@ -54,15 +54,15 @@ impl Data {
     let map_speedy_err = |p: Error| io::Error::new(io::ErrorKind::Other, p);
 
     let _extra_flags =
-      u16::read_from_stream_with_ctx(endianness, &mut cursor).map_err(map_speedy_err)?;
+      u16::read_from_stream_unbuffered_with_ctx(endianness, &mut cursor).map_err(map_speedy_err)?;
     let octets_to_inline_qos =
-      u16::read_from_stream_with_ctx(endianness, &mut cursor).map_err(map_speedy_err)?;
+      u16::read_from_stream_unbuffered_with_ctx(endianness, &mut cursor).map_err(map_speedy_err)?;
     let reader_id =
-      EntityId::read_from_stream_with_ctx(endianness, &mut cursor).map_err(map_speedy_err)?;
+      EntityId::read_from_stream_unbuffered_with_ctx(endianness, &mut cursor).map_err(map_speedy_err)?;
     let writer_id =
-      EntityId::read_from_stream_with_ctx(endianness, &mut cursor).map_err(map_speedy_err)?;
+      EntityId::read_from_stream_unbuffered_with_ctx(endianness, &mut cursor).map_err(map_speedy_err)?;
     let sequence_number =
-      SequenceNumber::read_from_stream_with_ctx(endianness, &mut cursor).map_err(map_speedy_err)?;
+      SequenceNumber::read_from_stream_unbuffered_with_ctx(endianness, &mut cursor).map_err(map_speedy_err)?;
 
     let expect_qos = flags.contains(DATA_Flags::InlineQos);
     let expect_data = flags.contains(DATA_Flags::Data) || flags.contains(DATA_Flags::Key);
@@ -80,7 +80,7 @@ impl Data {
 
     let parameter_list = if expect_qos {
       Some(
-        ParameterList::read_from_stream_with_ctx(endianness, &mut cursor)
+        ParameterList::read_from_stream_unbuffered_with_ctx(endianness, &mut cursor)
           .map_err(map_speedy_err)?,
       )
     } else {
