@@ -4,7 +4,7 @@ use speedy::{Readable, Writable};
 /// This message is sent from an RTPS Reader to an RTPS Writer.
 /// It contains explicit information on where to send a reply
 /// to the Submessages that follow it within the same message.
-#[derive(Debug, PartialEq, Readable, Writable)]
+#[derive(Debug, PartialEq, Clone, Readable, Writable)]
 pub struct InfoReply {
   /// Indicates an alternative set of unicast addresses that
   /// the Writershould use to reach the Readers when
