@@ -1,3 +1,4 @@
+use crate::structure::time::Timestamp;
 use crate::structure::guid::GUID;
 use crate::structure::sequence_number::SequenceNumber;
 //use crate::messages::submessages::submessage_elements::serialized_payload::SerializedPayload;
@@ -14,6 +15,7 @@ pub enum ChangeKind {
 pub struct CacheChange {
   pub writer_guid: GUID,
   pub sequence_number: SequenceNumber,
+  pub source_timestamp: Option<Timestamp>,
   pub data_value: DDSData, 
 }
 
@@ -22,6 +24,7 @@ impl PartialEq for CacheChange {
   fn eq(&self, other: &Self) -> bool {
       self.writer_guid == other.writer_guid
       && self.sequence_number == other.sequence_number
+      && self.source_timestamp == other.source_timestamp
       && self.data_value == other.data_value
   }
 }
@@ -30,9 +33,10 @@ impl CacheChange {
   pub fn new(
     writer_guid: GUID,
     sequence_number: SequenceNumber,
+    source_timestamp: Option<Timestamp>,
     data_value: DDSData,  
   ) -> CacheChange {
-    CacheChange { writer_guid, sequence_number, data_value, }
+    CacheChange { writer_guid, sequence_number, source_timestamp, data_value, }
   }
 
   pub fn change_kind(&self) -> ChangeKind {
