@@ -63,24 +63,24 @@ impl StatusInfo {
   pub fn change_kind(&self) -> ChangeKind {
     if self.contains(StatusInfoEnum::Disposed) {
       // DISPOSED is strongest
-      ChangeKind::NOT_ALIVE_DISPOSED
+      ChangeKind::NotAliveDisposed
     } else if self.contains(StatusInfoEnum::Unregistered) {
       // Checking unregistered second
-      ChangeKind::NOT_ALIVE_UNREGISTERED
+      ChangeKind::NotAliveUnregistered
     } else {
       // Even if filtered is set it is still alive
-      ChangeKind::ALIVE
+      ChangeKind::Alive
     }
   }
 
   pub fn into_cdr_bytes<BO: ByteOrder>(
     &self,
   ) -> Result<Vec<u8>, crate::serialization::error::Error> {
-    to_bytes::<StatusInfo, BO>(&self)
+    to_bytes::<StatusInfo, BO>(self)
   }
 
   pub fn from_cdr_bytes(
-    bytes: &Vec<u8>,
+    bytes: &[u8],
     representation_id: RepresentationIdentifier,
   ) -> Result<StatusInfo, crate::serialization::error::Error> {
     CDRDeserializerAdapter::from_bytes(bytes, representation_id)
